local addonName, TS = ...; TS.Logic = { }

--do not load the addon if current player is not a shaman
local a, b, classId = UnitClass("player")
if (classId ~= 7) then return end

local _showTotemExpiryAlarm = true
local ActiveTotems = 
{
	[1] = { Guid = "", SpellId = 0, StartTime = 0, Health = nil, AlarmColorFlag = false },
	[2] = { Guid = "", SpellId = 0, StartTime = 0, Health = nil, AlarmColorFlag = false },
	[3] = { Guid = "", SpellId = 0, StartTime = 0, Health = nil, AlarmColorFlag = false },
	[4] = { Guid = "", SpellId = 0, StartTime = 0, Health = nil, AlarmColorFlag = false }
}

--[[
	Determine if the totem with the specified totem index is active.
]]
local function IsTotemActive(totemIndex)
	return (ActiveTotems[totemIndex].StartTime ~= 0)
end

--[[
	Clear all data for the totem with the specified totem index.
]]
local function ClearTotem(totemIndex)
	ActiveTotems[totemIndex].Guid = ""
	ActiveTotems[totemIndex].SpellId = 0
	ActiveTotems[totemIndex].StartTime = 0
	ActiveTotems[totemIndex].Health = nil
	
	TS.UIElements[totemIndex].Timer:SetTextColor(1.0, 1.0, 1.0, 1.0)
	TS.UIElements[totemIndex].Image:SetTexture(nil)
	TS.UIElements[1].Health:SetMinMaxValues(0, 0)
	TS.UIElements[1].Health:SetValue(0)
	TS.Logic.UpdateTotemData()
end

--[[
	Get the time left for the totem with the specified totem index. 0 if totem not active.
]]
local function GetTimeLeft(totemIndex)
	if (not IsTotemActive(totemIndex)) then return nil end

	local startTime = ActiveTotems[totemIndex].StartTime
	local spellId = ActiveTotems[totemIndex].SpellId
	local duration = TS.Totems[spellId].Duration
	
	local timeElapsed = GetTime() - startTime
	local timeLeft = duration - timeElapsed
	if (timeLeft < 0) then timeLeft = 0 end
	
	return timeLeft
end

--[[
	Refreshes stoneclaw totem's health bar.
]]
local function RefreshHealth(totemIndex)
	if (totemIndex ~= 1 or ActiveTotems[1].SpellId == 0) then return end
	
	local spellId = ActiveTotems[1].SpellId
	local maxHealth = TS.Totems[spellId].Health
	if (maxHealth < 6) then return end
	
	TS.UIElements[1].Health:SetMinMaxValues(0, TS.Totems[spellId].Health)
	TS.UIElements[1].Health:SetValue(ActiveTotems[1].Health)
end

--[[
	Refreshes the time left until expiry of the totem with the given index.
]]
local function RefreshTimer(totemIndex)
	local timeLeftText = ""
	
	if (IsTotemActive(totemIndex)) 
	then
		local timeLeft = GetTimeLeft(totemIndex)
		
		local minutes = string.format("%2.f", math.floor(timeLeft / 60))
		local seconds = string.format("%02.f", math.floor(timeLeft  - minutes * 60))
		timeLeftText = minutes .. ":" .. seconds
	end
	
	TS.UIElements[totemIndex].Timer:SetText(timeLeftText)
end

--[[
	Get the number of players affected by the totem with the given index.
]]
local function GetAffectedCount(totemIndex, spellId)
	local unitsAffected = 0
	local allUsersAffected = true

	--get the number of units = 1 (player) + party members
	local unitCount = 1 + math.min(4, GetNumGroupMembers())
	for unitIndex = 1, unitCount 
	do
		--get ID of this unit and the totem aura (spell) ID we are looking for
		local unitId = TS.Utils.GetUnitId(unitIndex)
		local totemAuraId = TS.Totems[spellId].AuraId
		
		if (TS.Utils.CanUnitReceiveTotemBuffs(unitId))
		then
			if (TS.Utils.IsUnitAffected(unitId, totemAuraId)) 
			then 
				--if unit is affected by this totem, increment affected if yes
				unitsAffected = unitsAffected + 1 
			else
				--check if totem is useful for this class. if yes mark not all users as affected
				local isUseful = TS.Utils.IsAuraUsefulToUnit(unitId, totemAuraId)
				if (isUseful) then allUsersAffected = false end
			end
		end
	end
	
	return unitsAffected, allUsersAffected
end

--[[
	Refreshes the number of players affected by the totem with the given index.
]]
local function RefreshAffectedCount(totemIndex)
	local spellId = ActiveTotems[totemIndex].SpellId
	totemCountText = TS.UIElements[totemIndex].Count
	
	--if no totem found set empty text and return
	if (spellId == 0)
	then
		totemCountText:SetText("");
		return
	end
	
	--if affected count should be ignored for this totem set white text and —
	if (TS.Totems[spellId].AuraId == 0)
	then
		totemCountText:SetText("—");
		totemCountText:SetTextColor(1.0, 1.0, 1.0, 1.0)
	else
		local unitsAffected, allUsersAffected = GetAffectedCount(totemIndex, spellId)
		totemCountText:SetText(unitsAffected);

		if (allUsersAffected) 
		then 
			totemCountText:SetTextColor(0.0, 1.0, 0.0, 1.0)
		else
			totemCountText:SetTextColor(1.0, 0.0, 0.0, 1.0)
		end
	end
end

local function Alarm()
	for totemIndex = 1, 4 
	do
		local spellId = ActiveTotems[totemIndex].SpellId
		local isIgnored = TS.Utils.IsTotemIgnoredForAlarm(spellId)
	
		if (IsTotemActive(totemIndex) and GetTimeLeft(totemIndex) < 7 and not isIgnored) 
		then
			if (ActiveTotems[totemIndex].AlarmColorFlag == true)
			then
				TS.UIElements[totemIndex].Timer:SetTextColor(1.0, 1.0, 1.0, 1.0)
				ActiveTotems[totemIndex].AlarmColorFlag = false
			else
				TS.UIElements[totemIndex].Timer:SetTextColor(1.0, 0.0, 0.0, 1.0)
				ActiveTotems[totemIndex].AlarmColorFlag = true
			end
		end
	end
end

local function IsActiveTotemEvent(destGuid)
	if (destGuid == nil) then return false end
	
	return 
	(ActiveTotems[1].Guid == destGuid or 
	ActiveTotems[2].Guid == destGuid or 
	ActiveTotems[3].Guid == destGuid or 
	ActiveTotems[4].Guid == destGuid)
end

function TS.Logic.PlayerTotemUpdate(totemSlot)
	--in Blizzard's API earth and fire indexes are switched. switch them back here
	local totemIndex = totemSlot
	if (totemSlot == 1) then totemIndex = 2 end
	if (totemSlot == 2) then totemIndex = 1 end

	--if the last start time for this totem type is less than 0.75 sec ago from now ignore this event. this is necessary because sometimes PLAYER_TOTEM_UPDATE gets raised twice for the same totem event for no apparent reason
	local startTime = ActiveTotems[totemIndex].StartTime
	if (GetTime() < (startTime + 0.75)) then return end
	
	--if a totem currently exists for this slot then it must have been destroyed. clear data
	if (ActiveTotems[totemIndex].SpellId ~= 0) then ClearTotem(totemIndex) end
	
	TS.Logic.UpdateTotemData()
end

function TS.Logic.SpellCastSucceeded(guidData, spellId)
	local totemIndex = TS.Utils.GetSpellTotemIndex(spellId)
	if (totemIndex == nil) then return end
	
	--clear previous totem for this slot. PLAYER_TOTEM_UPDATE is not raised when one totem replaces another
	ClearTotem(totemIndex)
	
	--update active totems and totem data
	ActiveTotems[totemIndex].SpellId = spellId
	ActiveTotems[totemIndex].StartTime = GetTime()
	ActiveTotems[totemIndex].Health = TS.Totems[spellId].Health
	TS.Logic.UpdateTotemData()
end

--[[
	Update data (icon, expiry timer, affected count) for all totems.
]]
function TS.Logic.UpdateTotemData()
	for totemIndex = 1, 4 
	do
		--set icon
		local spellId = ActiveTotems[totemIndex].SpellId
		local icon = GetSpellTexture(spellId)
		if (spellId == 24854) then icon = GetItemIcon(20503) end	--Enamored Water Spirit
		TS.UIElements[totemIndex].Image:SetTexture(icon)
		
		--this clears totem data in the rare cases where PLAYER_TOTEM_UPDATE is not raised
		if ((GetTimeLeft(totemIndex) or 0) < 0) then ClearTotem(totemIndex) end
		
		--update expiry timer, health and affected count
		RefreshTimer(totemIndex)
		RefreshHealth(totemIndex)
		RefreshAffectedCount(totemIndex)
	end
end

--[[
	Updates all totem data at a certain interval.
]]
local _timeSinceLastUpdate = 0
local _timeSinceLastAlarmColorChange = 0
function TS.Logic.OnUpdate(elapsed)
	--update time since last update
    _timeSinceLastUpdate = _timeSinceLastUpdate + elapsed
	_timeSinceLastAlarmColorChange = _timeSinceLastAlarmColorChange + elapsed
		
	--change color for expiring totems every 0.3 sec
	if (_showTotemExpiryAlarm and _timeSinceLastAlarmColorChange > 0.3) 
	then
		Alarm()
		_timeSinceLastAlarmColorChange = 0
	end
	
	--update time left and affected count every 1 second
    if (_timeSinceLastUpdate > 1) 
	then
		TS.Logic.UpdateTotemData()
		_timeSinceLastUpdate = 0
	end
end

--[[
	Zone changed from/to an instance implies all totems destroyed. 
	For some reason PLAYER_TOTEM_UPDATE is not raised so this is a workaround.
]]
local _isPlayerInInstance = IsInInstance()
function TS.Logic.ZoneChanged()
	if (_isPlayerInInstance == IsInInstance()) then return end

	_isPlayerInInstance = IsInInstance()
	for totemIndex = 1, 4 
	do
		ClearTotem(totemIndex)
	end
end

--[[
	This is called when settings are changed and will set timer color to white to make 
	sure expiry alarm setting change is properly applied.
	Otherwise the timer could possibly get stuck with red text.
]]
function TS.Logic.ApplySettings(settings)
	_showTotemExpiryAlarm = settings.ShowTotemExpiryAlarm
	
	for totemIndex = 1, 4 
	do
		TS.UIElements[totemIndex].Timer:SetTextColor(1.0, 1.0, 1.0, 1.0)
	end
end

function TS.Logic.CombatLogEvent()
	--get event info and corresponding totem index, if not found then this event is not relevant for us
	local a, eventName, b, c, d, e, f, destGuid, destName, g, h, damage, i, j, spellDamage = CombatLogGetCurrentEventInfo()
	local totemIndex = TS.Utils.GetTotemIndexFromName(destName)
	if (totemIndex == nil) then return end
	
	--SPELL_SUMMON event is raised when totem is first created, set GUID
	if (eventName == "SPELL_SUMMON") then ActiveTotems[totemIndex].Guid = destGuid end
	
	--from here on disregard anything not related to an active totem
	if (not IsActiveTotemEvent(destGuid)) then return end
	
	--return if no damage was made, not relevant to an existing totem or totem health is nil
	if (damage == 0) then damage = spellDamage end
	if ((tonumber(damage) or 0) < 1 or ActiveTotems[totemIndex].Health == nil) then return end
	
	--subtract damage made from totem health
	ActiveTotems[totemIndex].Health = ActiveTotems[totemIndex].Health - damage
end